// *****************************************************************************
//
// Copyright. 2007. Symvionics, Inc. All rights reserved.
// 
// Symvionics Inc. ("Symvionics") grants a non-exclusive license for you to use
// and redistribute this software, in both source code and binary forms, and the
// accompanying documentation, if any, free of charge, provided that this
// paragraph, and the following two paragraphs appear in all copies and in all
// derivative works.  In no event shall this source code or this license thereto
// impact upon or fall within the scope of any otheragreement that you have with
// Symvionics.
// 
// BY USING THIS SOFTWARE, YOU FURTHER AGREE THAT, IN NO EVENT, SHALL SYMVIONICS
// BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
// SOFTWARE, THE ACCOMPANYING DOCUMENTATION, IF ANY, OR ANY DERIVATIVE WORKS.
// YOU AGREE TO DEFEND, INDEMNIFY, AND HOLD HARMLESS SYMVIONICS FROM AND AGAINST
// ALL FINES, PENALTIES, LIABILITIES, DAMAGES, COSTS, AND EXPENSES INCURRED
// BY SYMVIONICS AS A RESULT OF YOUR USE OF THIS SOFTWARE INCLUDING THIRD PARTY
// USE OF ANY DERIVATIVE WORKS CREATED BY YOU.
// 
// SYMVIONICS SPECIFICALLY DISCLAIMS ANY AND ALL WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE.  THE SOFTWARE AND ACCOMPANYING DOCUMENTATION LICENSED
// HEREUNDER ARE PROVIDED "AS IS" WITHOUT WARRANTY OR OBLIGATION OF SUPPORT OR 
// MAINTENANCE.
//
// *****************************************************************************

// SimpleFunction.h : Declaration of the CSimpleFunction

#ifndef __SIMPLEFUNCTION_H_
#define __SIMPLEFUNCTION_H_

#include "resource.h"       // main symbols
#include "IadsFunction.h"

/////////////////////////////////////////////////////////////////////////////
// CSimpleFunction
class ATL_NO_VTABLE CSimpleFunction : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CSimpleFunction, &CLSID_SimpleFunction>,
  public ISimpleFunction,
  public IIadsFunction
{
public:
	CSimpleFunction()
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_SIMPLEFUNCTION)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CSimpleFunction)
	COM_INTERFACE_ENTRY(ISimpleFunction)
	COM_INTERFACE_ENTRY(IIadsFunction)
END_COM_MAP()

// ISimpleFunction
public:


  STDMETHOD(FinalConstruct)( void )
  {
    // See SimpleFunction2.h for discussion on this function
    return S_OK;
  }

  STDMETHOD(FinalRelease)( void )
  {
    // See SimpleFunction2.h for discussion on this function
    return S_OK;
  }
  
  STDMETHOD(Compute)( /*[in]*/ VARIANT* dataIn, /*[out]*/ VARIANT* dataOut )
  {
    // About this function
    // FunctionName in Iads -> SampleFunctionVC.SimpleFunction  (Basically, this is the "ProgId" for this class)
    // How to call in Iads  -> SampleFunctionVC.SimpleFunction( Parameter1, Parameter2, Parameter3 )   (You can also feed in constant values) 
    // In Args              -> FloatValue1, FloatValue2, FloatValue3
    // Out Args             -> FloatValueSum

    // Background info on the Compute function interface
    // The VARIANT "dataIn" argument contains a "SafeArray" of input arguments passed from the user's
    // function (from of their derived equation) into this function. It will contain all the values
    // that you'll need to compute the function result. The result is returned in the "dataOut" argument.
    // The "dataOut" arguement is a VARIANT so we can return numeric results, as well as strings and other types
    // For a more advanced example on returning strings and other types, see the SimpleFunction2 class in this project
    // This function will be called once per data point calculated

    // First Step: We need to check how many arguments were passed into our function from the user
    // Example, if the user creates a derived parameter with the function -> SampleFunction.SimpleFunction( "Format", ValueOrParam1, ValueOrParam2 )
    // The argument count would be 3
    int argCount = dataIn->parray->rgsabound->cElements;
    if ( argCount != 3 )
    {
      return DISP_E_BADPARAMCOUNT;
    }

    // Now, get the input arguments array
    VARIANT* argsArray = (VARIANT*)(dataIn->parray->pvData);  // Could use SafeArrayAccessData, but slow..

    // Second Step: Check Types of each arg..... Either VT_R8 (floating point value), VT_BSTR (string value) for now...
    // In IADS, most every type of numerical argument is passed via an 8 byte floating point value VT_R8. 
    // If you're in doubt, use VT_R8. You can also break here in the code and examine the argsArray[N].vt value to see the actual type 
    // Example, if the user creates a derived parameter with the function -> SampleFunction.SimpleFunction( ValueOrParam1, ValueOrParam2, ValueOrParam3 )
    // The argument type for every argument would be VT_R8    
    if ( argsArray[0].vt != VT_R8 ) return E_INVALIDARG;
    if ( argsArray[1].vt != VT_R8 ) return E_INVALIDARG;
    if ( argsArray[2].vt != VT_R8 ) return E_INVALIDARG;

    // Third step: Get the actual values of each arg by extracting from the array of input arguments
    register double p1 = argsArray[0].dblVal;
    register double p2 = argsArray[1].dblVal;
    register double p3 = argsArray[2].dblVal;

    // Final step: Perform your function's purpose and return the output value
    // Add em up and ship it out... Because we're returning a number, the return type is VT_R8 (double) for now... Iads will convert if necessary..
    dataOut->vt = VT_R8;
    dataOut->dblVal = p1 + p2 + p3;

    return S_OK;
  }


};

#endif //__SIMPLEFUNCTION_H_
